#!/bin/bash

set -e
set -o pipefail

GEN='pypy ../generators/gen/random_generator.py'
SOLVER='pypy ../submissions/accepted/rgl.py'
TCID=1

testcase() {
  generator="$1"
  name="$2"
  ${generator} --name="${name}" "${@:3}" \
   > "$(printf 'secret/%03d-%s.in' "${TCID}" "${name}")" \
   &

  sleep 0
  ((TCID++))
  if { ( ((TCID%5 == 0)) ) } then wait; fi
}

echo "Creating secret data directory"
mkdir -p secret
find secret -mindepth 1 -not -name '*hand*' -delete

testcase "${GEN}" uniform-1 --buckets=4
testcase "${GEN}" uniform-2 --buckets=10
testcase "${GEN}" uniform-3 --buckets=20
testcase "${GEN}" uniform-huge-4 --buckets=100000
for i in {10..40} ; do
  testcase "${GEN}" uniform-$i
done
for i in {41..45} ; do
  testcase "${GEN}" uniform-large-$i --buckets=20000
done
for i in {46..50} ; do
  testcase "${GEN}" uniform-huge-$i --buckets=100000
done

## Solve all the cases

for test_in in secret/*.in
  do
    echo "Solving case ${test_in}"
    test_ans="$(dirname "${test_in}")"/"$(basename "${test_in}" ".in").ans"
    ${SOLVER} <"${test_in}" 1>"${test_ans}" 2>/dev/null &
  done
