#include <unordered_map>/*{{{*/
#include <algorithm>
#include <iostream>
#include <fstream>
#include <sstream>
#include <cstdint>
#include <climits>
#include <cassert>
#include <utility>
#include <string>
#include <vector>
#include <stack>
#include <cmath>
#include <map>
#include <set>
using namespace std;/*}}}*/

enum JudgeCode:int{
  judge_error=1,
  correct=42,
  wrong_answer=43,
};

struct Bucket{
  long double angle;
  int64_t empty;
  int64_t full;
};

long double get_cx(vector<Bucket> const &buckets, long double angle){
  long double cx=0.L;
  for (int i=0; i<buckets.size(); i++){
    long double x=sinl((angle+buckets[i].angle)*M_PI/180.L);
    cx+=x*(x>=0? buckets[i].full: buckets[i].empty);
  }
  return cx;
}

////////////////////////////////////////////////////////////////////

int main(int argc, char **argv){
  if (argc<3){
    cerr<<"usage: "<<argv[0]<<" testcase.in testcase.ans [feedback_dir] < team.out"<<endl;
    return judge_error;
  }else if (argc<4){
    cerr<<"writing feedback to stdout"<<endl;
  }else{
    if (argc>4){
      cerr<<"warning: ignoring "<<argc-4<<" extra arguments"<<endl;
    }
    cerr<<"writing feedback to "<<argv[3]<<"/feedback.txt"<<endl;
    if (not freopen((string(argv[3]) + "/feedback.txt").c_str(), "a", stdout)){
      cerr<<"unable to open feedback file"<<endl;
      return judge_error;
    }
  }

  ifstream canonical_input(argv[1]);
  ifstream canonical_output(argv[2]);
  auto &team_output = cin;
  cout.precision(10);

  // Read canonical input.
  int n; assert(canonical_input>>n);
  vector<Bucket> buckets(n);
  for (int i=0; i<n; i++){
    assert(canonical_input>>buckets[i].angle>>buckets[i].empty>>buckets[i].full);
  }

  // Read canonical answer.
  long double canonical_answer; assert(canonical_output>>canonical_answer);

  // Read team answer.
  long double team_answer;
  if (not (team_output>>team_answer)){
    cout<<"Team failed to print anything meaningful"<<endl;
    return wrong_answer;
  }

  long double canonical_cx=get_cx(buckets,canonical_answer);
  long double team_cx=get_cx(buckets,team_answer);

  // FIXME the tolerance needs to be based on distance from correct angle,
  // not on distance from correct centre of mass.
  long double error=abs(canonical_cx-team_cx);
  long double largest=max(canonical_cx,team_cx);

  if (error<2*1e-6L or (error/max(0.0L,largest))<2*1e-6L){
    cout<<"OK team answer cx="<<team_cx<<", our answer cx="<<canonical_cx<<endl;
    return correct;
  }else{
    cout<<"Fail team answer cx="<<team_cx<<", our answer cx="<<canonical_cx<<endl;
    return team_cx < canonical_cx? wrong_answer: judge_error;
  }
}
