#!/usr/bin/env python

"""
run to create the figure on the problem statement.
the input used is the first line of ../data/sample/1.in
"""

from math import sin,cos,pi,hypot

def get_xy(ang, radius):
  return (
    radius * sin(ang*pi/180.0),
    radius * cos(ang*pi/180.0))

if __name__ == '__main__':
  f = open('../data/sample/1.in', 'r')
  ff = map(lambda s: s.strip(), f.readlines())
  s = [
    '\\documentclass[class=article,border=0pt]{standalone}\n',
    '\\usepackage[usenames,dvipsnames]{xcolor}\n',
    '\\usepackage{tikz}\n',
    '\\begin{document}\n',
    '\\pagestyle{empty}\n',
    '\\thispagestyle{empty}\n',
    '\\begin{tikzpicture}[scale=0.25]\n']
  y0 = 0
  k = 0

  inner_radius = 5
  outer_radius = 10
  drawn_angle_offset = 15

  s.append('\\draw[step=10mm, line width=0.05mm, black!15!white] (-{0}-.01,-{1}-.01) grid ({0}+.01,{1}+.01);'.format(outer_radius*4, outer_radius*1.5))
  s.append('\\draw[step=50mm, line width=0.1mm, black!20!white] (-{0}-.01,-{1}-.01) grid ({0}+.01,{1}+.01);'.format(outer_radius*4, outer_radius*1.5))
  s.append('\\draw[step=500mm, line width=0.1mm, black!25!white] (-{0}-.01,-{1}-.01) grid ({0}+.01,{1}+.01);'.format(outer_radius*4, outer_radius*1.5))
  s.append('\\draw[step=100mm, line width=0.15mm, black!45!white] (-{0}-.01,-{1}-.01) grid ({0}+.01,{1}+.01);'.format(outer_radius*4, outer_radius*1.5))

  # Draw the outer wheel.
  s.append('\\path [draw=black,fill=brown,fill opacity=1,even odd rule] (0,0) circle ({0}) (0,0) circle ({1});'.format(outer_radius-0.5, outer_radius+0.5))

  # Draw the inner wheel.
  s.append('\\path [draw=black,fill=brown,fill opacity=1,even odd rule] (0,0) circle ({0}) (0,0) circle ({1});'.format(0.25, 1.0))
  s.append('\\draw [fill=white] ({0},{1}) ellipse ({2} and {2});\n'.format(
      0, 0, 0.25))

  centre_of_mass = [0, 0]
  for angle, empty, full in map(lambda i: map(float, i.split()), ff[1:]):
    angle += drawn_angle_offset

    while angle < 0: angle += 360
    while angle >= 360: angle -= 360

    stem = get_xy(angle, inner_radius)
    stern = get_xy(angle, outer_radius)

    def fix_path(tilt, centre, move_to, *args):
      args = [i for i in args]
      for i in range(0, len(args)-1, 2):
        args[i], args[i+1] = args[i] - centre[0], args[i+1] - centre[1]
        args[i], args[i+1] = args[i]*cos(tilt) + args[i+1]*sin(tilt), args[i]*-sin(tilt) + args[i+1]*cos(tilt)
        args[i], args[i+1] = args[i] + move_to[0], args[i+1] + move_to[1]
      return args

    filled = (0 <= angle < 180)
    tilt = (0 if filled else -pi*3/7)

    # Draw the attachment point.
    s.append('\\draw [fill=white] ({0},{1}) ellipse ({2} and {2});\n'.format(
        stern[0], stern[1], 0.25))

    # Draw the cradle.
    if filled:
      attachment_point = centre_point = (stern[0] + 1, stern[1] - 2)
      s.append('\\path [draw=black] ({0},{1}) to [bend right=10] ({2},{3}) to [bend right=10] ({4},{5});\n'.format(
          stern[0] - 1, stern[1] - 2, stern[0], stern[1], stern[0] + 1, stern[1] - 2))
    else:
      centre_point = (stern[0] + 1, stern[1] - 2)
      attachment_point = (stern[0] - 0.5, stern[1] - 1.18)
      s.append('\\path [draw=black] ({0},{1}) to [bend left=2] ({2},{3}) to [bend right=10] ({0},{1});\n'.format(
          stern[0], stern[1], *attachment_point))

    # Draw the bucket.
    s.append('\\path [fill=lightgray,draw=black] ({0},{1}) to ({2},{3}) to ({4},{5}) to ({6},{7}) to [bend left=10] ({0},{1});\n'.format(
        *fix_path(tilt, centre_point, attachment_point,
            stern[0] - 1.1, stern[1] - 2,
            stern[0] - 0.6, stern[1] - 4,
            stern[0] + 0.6, stern[1] - 4,
            stern[0] + 1.1, stern[1] - 2)))

    s.append('\\path [fill=brown,draw=black] ({0},{1}) to ({2},{3}) to ({4},{5}) to ({6},{7}) to ({8},{9}) to ({10},{11}) to ({12},{13}) to ({14},{15}) to ({0},{1});\n'.format(
        *fix_path(tilt, centre_point, attachment_point,
            stern[0] - 1.1, stern[1] - 2,
            stern[0] - 0.6, stern[1] - 4,
            stern[0] + 0.6, stern[1] - 4,
            stern[0] + 1.1, stern[1] - 2,
            stern[0] + 0.9, stern[1] - 2,
            stern[0] + 0.4, stern[1] - 3.8,
            stern[0] - 0.4, stern[1] - 3.8,
            stern[0] - 0.9, stern[1] - 2)))

    if filled:
      # Draw the water.
      s.append('\\path [fill=blue,draw=black] ({0},{1}) to ({2},{3}) to ({4},{5}) to ({6},{7});\n'.format(
        stern[0] - 0.75, stern[1] - 2.5,
        stern[0] - 0.4, stern[1] - 3.8,
        stern[0] + 0.4, stern[1] - 3.8,
        stern[0] + 0.75, stern[1] - 2.5))

    # Update centre of mass.
    centre_of_mass[0] += stern[0] * (full if filled else empty)
    centre_of_mass[1] += stern[1] * (full if filled else empty)

  # Draw the centre of mass.
  s.append('\\draw [draw=black,->] (0,0) to ({0},{1});\n'.format(*centre_of_mass))

  s.append('\\end{tikzpicture}\n')
  s.append('\\end{document}\n')
  f = open('fig.tex', 'w')
  for x in s:
    f.write(x)
  f.close()
