#include <unordered_map>/*{{{*/
#include <algorithm>
#include <iostream>
#include <fstream>
#include <sstream>
#include <climits>
#include <cassert>
#include <utility>
#include <string>
#include <vector>
#include <stack>
#include <map>
#include <set>
using namespace std;/*}}}*/

enum JudgeCode:int{
  judge_error=1,
  correct=42,
  wrong_answer=43,
};

////////////////////////////////////////////////////////////////////

#define V_MAX 100001
#define E_MAX 500001

struct TestCase{
  int n;
  vector<int> types;
  vector<int> parents;

  TestCase(istream &f){
    assert(f>>n);
    types.assign(n,-1);
    parents.assign(n,-1);

    for (int i=0; i<n; i++){
      assert(f>>types[i]>>parents[i]);
      --parents[i];
      if (parents[i]==i){
        parents[i]=-1;
      }
    }
  }
};

struct CheckAssignment{
  TestCase const &test;
  JudgeCode const on_fail;

  set<int> assigned;

  void assign_person(int a,bool really=true){
    if (a<0 or a>=test.n){
      cout<<"Person "<<a+1<<" is out of bounds"<<endl;
      exit(on_fail);
    }
    if (assigned.count(a)){
      cout<<"Assigned "<<a+1<<" twice"<<endl;
      exit(on_fail);
    }
    if (really){
      assigned.insert(a);
    }
  }

  void add_edge(int a,int b){
    assign_person(a,false);
    assign_person(b,false);
    if (test.types[a]!=1){
      cout<<"Used a short person on the first row"<<endl;
      exit(on_fail);
    }
    if (test.types[b]!=0){
      cout<<"Used a tall person on the second row"<<endl;
      exit(on_fail);
    }
    if (test.types[a]==test.types[b]){
      cout<<"Items "<<a+1<<" and "<<b+1<<" are same type"<<endl;
      cout<<"Failing with code "<<on_fail<<endl;
      exit(on_fail);
    }
    if (test.parents[a]==b or test.parents[b]==a){
      cout<<"Items "<<a+1<<" and "<<b+1<<" are directly linked"<<endl;
      cout<<"Failing with code "<<on_fail<<endl;
      exit(on_fail);
    }
    assign_person(a);
    assign_person(b);
  }

  explicit CheckAssignment(string first,istream &f,TestCase const &tc,JudgeCode on_fail): test(tc), on_fail(on_fail){
    vector<int> numbers(test.n);
    for (int i=0; i<test.n; i++){
      if (i==0){
        stringstream sst(first);
        if (not (sst>>numbers[i])){
          cout<<"Output did not start with a number."<<endl;
          exit(on_fail);
        }
      }
      else if (not (f>>numbers[i])){
        cout<<"Output finished too early."<<endl;
        exit(on_fail);
      }
    }

    for (int i=0; i<test.n/2; i++){
      add_edge(numbers[i]-1,numbers[i+test.n/2]-1);
    }
  }
};

////////////////////////////////////////////////////////////////////

int main(int argc, char **argv){
  if (argc<3){
    cerr<<"usage: "<<argv[0]<<" testcase.in testcase.ans [feedback_dir] < team.out"<<endl;
    return judge_error;
  }else if (argc<4){
    cerr<<"writing feedback to stdout"<<endl;
  }else{
    if (argc>4){
      cerr<<"warning: ignoring "<<argc-4<<" extra arguments"<<endl;
    }
    cerr<<"writing feedback to "<<argv[3]<<"/feedback.txt"<<endl;
    if (not freopen((string(argv[3]) + "/feedback.txt").c_str(), "a", stdout)){
      cerr<<"unable to open feedback file"<<endl;
      return judge_error;
    }
  }

  ifstream canonical_input(argv[1]);
  ifstream canonical_output(argv[2]);
  auto &team_output = cin;

  // Read canonical input.
  TestCase tc(canonical_input);

  // Read canonical answer.
  string canonical_answer;
  if (not (canonical_output>>canonical_answer)){
    cout<<"Canonical output is empty"<<endl;
    return judge_error;
  }

  // Read team answer.
  string team_answer;
  if (not (team_output>>team_answer)){
    cout<<"Team failed to print anything"<<endl;
    return wrong_answer;
  }

  bool const impossible=(canonical_answer=="impossible");
  if (not impossible){
    CheckAssignment(team_answer,team_output,tc,wrong_answer);
    CheckAssignment(canonical_answer,canonical_output,tc,judge_error);
    cout<<"OK team found a correct answer"<<endl;
    return correct;
  }

  cout<<"Data set is impossible"<<endl;

  if (team_answer=="impossible"){
    cout<<"OK team printed \"impossible\""<<endl;
    return correct;
  }else{
    cout<<"Team did NOT print \"impossible\"."<<endl;
    // Fall through and let the validator print out what they did wrong.
  }
  CheckAssignment(team_answer,team_output,tc,wrong_answer);
  cout<<"Team found a correct answer, but case is supposed to be impossible"<<endl;
  return judge_error;
}
