#!/usr/bin/env python
from sys import argv

import svgwrite

author_input = argv[1]
team_answer = argv[2] if len(argv) >= 3 else None

print ' +======/'
print '|| Reading input from', author_input, 'and answer from', team_answer
print ' +======\\'

with open(author_input) as f:
  building_sites = [map(float, f.readline().split()) for _ in range(3)]
  trees_n = int(f.readline().split()[0])
  trees = map(float, f.readline().split())
  assert trees_n == len(trees)

team_locations = []
if team_answer is not None:
  with open(team_answer) as f:
    possible = (f.readline().split()[0] == 'possible')
    if possible:
      team_locations = [map(float, f.readline().split()) for _ in trees]

min_x = min(*map(lambda _: _[0], building_sites + team_locations))
min_y = min(*map(lambda _: _[1], building_sites + team_locations))
max_x = max(*map(lambda _: _[0], building_sites + team_locations))
max_y = max(*map(lambda _: _[1], building_sites + team_locations))

svg = svgwrite.Drawing(
  filename="testcase.svg",
  size = ("820px", "820px"))

span = max(max_x-min_x, max_y-min_y, 1.0)

def normalise(p):
  return [(p[0]-min_x) / span * 800 + 10,
          (p[1]-min_y) / span * 800 + 10]

def add_rect(p,r):
  p = normalise(p)
  svg.add(svg.rect(
    insert = (p[0]-r, p[1]-r),
    size = (r*2, r*2),
    stroke_width = "1",
    stroke = "black",
    fill = "rgb(255,255,0)"))

def add_circle(p,r):
  p = normalise(p)
  r = r / span * 800
  svg.add(svg.circle(
    center = (p[0], p[1]),
    r = r,
    stroke_width = "1",
    stroke = "black",
    fill = "rgb(200,200,200)"))

for p,r in zip(team_locations, trees):
  add_circle(p, r)

for p in building_sites:
  add_rect(p, 5)

svg.save()
