#!/usr/bin/env python
from math import hypot
from sys import exit

debug = False
do_print_answer = False

# Define some geom functions.
def vsub(a, b): return [a[0]-b[0], a[1]-b[1]]
def vadd(a, b): return [a[0]+b[0], a[1]+b[1]]
def vmul(a, b): return [a[0]*b, a[1]*b]
def vdot(a, b): return a[0]*b[0] + a[1]*b[1]
def vcrz(a, b): return a[0]*b[1] - a[1]*b[0]
def vlen(a): return hypot(*a)
def vnml(a):
  l = vlen(a)
  return [0.0, 0.0] if l < 1e-9 else [a[0]/l, a[1]/l]

# Define a combinatoric function.
def generate_partitions(v, callback, i=0, r=[0, 0, 0], assignments = None):
    if assignments is None:
      assignments = [0 for _ in v]

    if i == len(v):
      callback(r, assignments)
      return

    for j in range(3):
      assignments[i] = j
      r[j] += v[i][0]
      generate_partitions(v, callback, i+1, r, assignments)
      r[j] -= v[i][0]

# Define another geom function.
def radical_centre(a,ar,b,br,c,cr):
    b = vsub(b,a)
    c = vsub(c,a)
    if abs(vcrz(b,c)) < 1e-9: return None
    u = ar*ar-cr*cr+vdot(c,c)
    v = ar*ar-br*br+vdot(b,b)
    w = vcrz(c,b)*2
    return vadd(a, [(b[1]*u-c[1]*v)/w, -(b[0]*u-c[0]*v)/w])

### Quick unit test for radical_centre
if debug:
    def check_dist(a, b):
        if vlen(vsub(a,b)) > 1e-9:
            print 'check_dist (%.3f,%.3f), (%.3f,%.3f)' % (a[0],a[1],b[0],b[1])
            assert False
    check_dist(radical_centre([0.0,1.0], 1.0, [2.0,0.0], 1.0, [2.0,2.0], 1.0), [1.25, 1.0])
    check_dist(radical_centre([0.0,1.0], 1.0, [2.0,0.0], 3.0, [2.0,2.0], 1.0), [0.25, 3.0])
###

def test_partition(p, assignments):
    global hub, place
    radius = map(lambda i: i*2+hub[0], p)
    centre = radical_centre(place[0], radius[0], place[1], radius[1], place[2], radius[2])
    test_centre(centre, p, assignments)

def test_centre(centre, p, assignments):
    global hub, place
    radius = map(lambda i: i*2+hub[0], p)
    for hxy in ([centre] + place if centre is not None else []):
        ok = True
        for p,r in zip(place, radius):
            ok = ok and vlen(vsub(hxy,p)) <= r + 1e-9
        if ok:
            print_answer(hxy, p, assignments)

def print_answer(centre, p, assignments):
    print 'possible'
    if do_print_answer:
        global trees
        diren = [vnml(vsub(_,centre)) for _ in place]
        where = [hub[0] for _ in range(3)]
        answer = [[0.0, 0.0] for _ in range(n)]
        if hub[1] is not None:
            answer[hub[1]] = centre
        for j in range(len(assignments)-1,-1,-1):
            i = assignments[j]
            tr = trees[j][0]
            where[i] += tr
            answer[trees[j][1]] = vadd(centre, vmul(diren[i], where[i]))
            where[i] += tr
        for i in answer:
            print '%.8f %.8f' % (i[0], i[1])
    exit(0)

def solution():
    global hub, place, trees, n

    # Take in the list of places.
    place = sorted(map(lambda i: map(float, raw_input().split()), range(3)))

    # Take in the list of trees.
    n = int(raw_input())
    trees = sorted(zip(map(float, raw_input().split()), range(n)))

    # Check if the 3 places are colinear.
    if abs(vcrz(vsub(place[2], place[1]), vsub(place[1], place[0]))) < 1e-9:
        hub = [0.0, None]
        if vlen(vsub(place[2], place[0])) < 2*sum(map(lambda _: _[0], trees)) + 1e-9:
          print_answer(
            place[2],
            [sum(map(lambda _: _[0], trees)), 0, 0],
            [0 for _ in trees])
    else:
        # Use the biggest tree as a hub.
        hub = trees.pop()
        generate_partitions(trees, test_partition)

    print 'impossible'

solution()


