#include <bits/stdc++.h>
using namespace std;
typedef long double ll;

constexpr bool debug=false;

struct vec2{/*{{{*/
  ll x,y;
  bool operator==(vec2 const &a)const{return x==a.x and y==a.y;}
  vec2 &operator*=(ll const a){x*=a; y*=a; return *this;}
  vec2 operator*(ll const a)const{return {x*a,y*a};}
  vec2 operator/(ll const a)const{return {x/a,y/a};}
  vec2 operator+(vec2 const &a)const{return {x+a.x,y+a.y};}
  vec2 operator-(vec2 const &a)const{return {x-a.x,y-a.y};}
  bool operator<(vec2 const &a)const{return x!=a.x? x<a.x: y<a.y;}
  ll dot(vec2 const &a)const{return x*a.x+y*a.y;}
  ll cross(vec2 const &a)const{return x*a.y-y*a.x;}
  ll length()const{return hypotl(x,y);}
  vec2 &normalise(){ll len=length(); if (len) x/=len,y/=len; return *this;}
};

istream &operator>>(istream &is, vec2 &p){
  return is>>p.x>>p.y;
}

template <typename A, typename B> istream &operator>>(istream &is, pair<A,B> &p){
  return is>>p.first>>p.second;
}

template <typename C> istream &operator>>(istream &is, vector<C> &v){
  size_t n; is>>n; v.resize(n);
  for (auto &i: v) is>>i;
  return is;
}/*}}}*/

template <size_t P>/*{{{*/ vector<array<ll,P>> generate_partitions(
    vector<ll>::const_iterator s,
    vector<ll>::const_iterator t){
  if (s==t) return {array<ll,P>()};
  auto res=generate_partitions<P>(s+1,t);
  for (size_t i=res.size(); i-->0;){
    for (size_t j=P; j-->1;){
      res.emplace_back(res[i]);
      res.back()[j]+=*s;
    }
    res[i][0]+=*s;
  }
  return res;
}/*}}}*/

template <class C> class optional{/*{{{*/
  private:
    C const *const value;

  public:
    optional(): value(nullptr) {}
    optional(C const v): value(new C(v)) {}
    optional(optional<C> const &other) = delete;
    optional(optional<C> const &&other): value(other.value) {}
    virtual ~optional() {if (value) delete value;}

    C const &value_or(C const fallback) {return value ? *value : fallback;}
};/*}}}*/

optional<vec2> radical_centre(/*{{{*/
    vec2 a, ll ar,
    vec2 b, ll br,
    vec2 c, ll cr){
  b=b-a;
  c=c-a;
  if (fabsl(b.cross(c)) < 1e-9L){
    return {};
  }
  /*
    1) we're looking for a point with the same power across all circles
    ===
    x^2+y^2 - ar^2 =
    x^2 - 2x*bx + bx^2 + y^2 - 2y*by + by^2 - br^2 =
    x^2 - 2x*cx + cx^2 + y^2 - 2y*cy + cy^2 - cr^2

    2) remove duplicate terms
    ===
    -ar^2 =
    -2x*bx + bx^2 - 2y*by + by^2 - br^2 =
    -2x*cx + cx^2 - 2y*cy + cy^2 - cr^2

    3) make into two simultaneous equations, multiply by cx and bx
    ===
    -ar^2*cx = -2x*bx*cx + cx*bx^2 - 2y*by*cx + cx*by^2 - cx*br^2
    -ar^2*bx = -2x*cx*bx + bx*cx^2 - 2y*cy*bx + bx*cy^2 - bx*cr^2

    4) remove duplicate terms
    ===
    -ar^2*cx = cx*bx^2 - 2y*by*cx + cx*by^2 - cx*br^2
    -ar^2*bx = bx*cx^2 - 2y*cy*bx + bx*cy^2 - bx*cr^2

    5) subtract the equations from each other
    ===
    ar^2*(bx-cx) + 2y*(by*cx-cy*bx) =
      + cx*bx^2 - bx*cx^2
      + cx*by^2 - bx*cy^2
      - cx*br^2 + bx*cr^2

    6) rearrange to get y
    ===
    y =-(bx*ar^2 - cx*ar^2
       + bx*cx^2 - cx*bx^2
       + bx*cy^2 - cx*by^2
       - bx*cr^2 + cx*br^2
       ) / 2(by*cx-cy*bx)
  */
  auto const u=ar*ar-cr*cr+c.dot(c);
  auto const v=ar*ar-br*br+b.dot(b);
  auto const w=c.cross(b)*2;
  return a+vec2{+(b.y*u-c.y*v),-(b.x*u-c.x*v)}/w;
}/*}}}*/

int main(){
  // Take in the list of places and the list of trees, both in sorted order.
  vec2 place[3]; for (auto &i: place) cin>>i; sort(place,place+3);
  vector<ll> trees; cin>>trees; sort(trees.begin(),trees.end());

  // Are all 3 places colinear? If so, shortcut.
  if (fabsl((place[1]-place[0]).cross(place[2]-place[0]))<1e-9L){
    ll const need=(place[2]-place[0]).length();
    puts(need <= 2*accumulate(trees.begin(),trees.end(),0.0L)+1e-9L? "possible": "impossible");
    return 0;
  }

  // Choose the biggest tree to use for the hub.
  ll const hub=trees.back();
  trees.pop_back();

  puts([&place,&trees,&hub](){
    // Partition the trees into 3 sets (each set = one edge from the hub).
    for (auto const p: generate_partitions<3>(trees.begin(), trees.end())){
      ll const radius[3]={p[0]*2+hub,p[1]*2+hub,p[2]*2+hub};
      auto centre=radical_centre(
          place[0], radius[0],
          place[1], radius[1],
          place[2], radius[2]).value_or(place[1]);
      if (debug){/*{{{*/
        cerr<<"p = "<<p[0]<<" "<<p[1]<<" "<<p[2]<<endl;
      }/*}}}*/
      for (vec2 const z: {centre,place[0],place[1],place[2]}){
        if (debug){/*{{{*/
          cerr<<"  candidate hub: "<<z.x<<" "<<z.y<<endl;
          for (int i=0; i<3; i++){
            cerr<<"    "<<hypotl(z.x-place[i].x,z.y-place[i].y)<<" "<<radius[i]<<endl;
          }
        }/*}}}*/
        static constexpr int numbers[3] = {0,1,2};
        if (accumulate(numbers,numbers+3,true,[z,&place,&radius](bool a,int i){
          return a and (z-place[i]).length() <= radius[i] + 1e-9L;
        })) return true;
      }
    }
    return false;
  }()? "possible": "impossible");
}
