#include <iostream>
#include <queue>
#include <vector>
#include <array>
#include <algorithm>
#include <climits>
#include "math.h"
using namespace std;

struct Edge;

typedef struct Cafe {
	int opens;
	int closes;
	uint64_t w;
	vector<int> distances;
	vector<Edge *> incoming;
} Cafe;

typedef struct Edge {
	int left;
	int arrive;
	int from;
	int to;
	uint64_t data;
	int id;
	Edge(int l, int a, int f, int t, int d, int id):left(l),arrive(a),from(f),to(t),data(d),id(id) {};
} Edge;


bool compare_edges(Edge *a, Edge *b) {
	return a->arrive < b->arrive;
}

int main(){
	int d; cin >>d; // size of the data in Mb.
	int n; cin >>n; // number of cafes

	vector<Cafe> cafes;
	vector<Edge *> edges;
	for (int i = 0; i < n; ++i) {
		Cafe new_cafe;
		cin>>new_cafe.opens;
		cin>>new_cafe.closes;
		cin>>new_cafe.w;
		// "We also create a connection to each shop with 0 data at the time they are open."
		auto edge = new Edge(new_cafe.opens,new_cafe.opens,-1,i,0,i);
		edges.push_back(edge);
		new_cafe.incoming.push_back(edge);
		//cout<<"new cafe incoming="<<new_cafe.incoming.size()<<endl;
		cafes.push_back(new_cafe);
	};
	for (int i = 0; i < n; ++i) {
		for (int j = 0; j < n; ++j) {
			int d; cin>>d;
			cafes[i].distances.push_back(d);
		}
		//cout<<"cafe "<<i<<" incoming.size()="<<cafes[i].incoming.size()<<endl;
	}

	// "So, for each shop we build a list of connections to other shops that will open
	// (if they open during the current shop's opening hours) AND all shops that 
	// we can travel to when our shop shuts (if we have to wait outside that is fine)"
	for (int i = 0; i < n; ++i) {
		for(int j = 0; j < n; ++j) {
			if (i==j) {
				continue;
			}
			if (cafes[j].opens >= (cafes[i].opens + cafes[i].distances[j])
				&& cafes[j].opens <= (cafes[i].closes + cafes[i].distances[j])) {
				auto edge = new Edge(
						cafes[j].opens - cafes[i].distances[j],
						cafes[j].opens,
						i,
						j,
						0,
						edges.size()
					);
				//cout<<"we can leave cafe "<<i<<" to go to cafe "<<j<<endl;
				edges.push_back(edge);
				cafes[j].incoming.push_back(edge);
			}
			if (cafes[j].closes >= (cafes[i].closes + cafes[i].distances[j])) {
				auto edge = new Edge(
						cafes[i].closes,
						max(cafes[i].closes + cafes[i].distances[j], cafes[j].opens),
						i,
						j,
						0,
						edges.size()
					);
				//cout<<"when cafe "<<i<<" closes, we can go to cafe "<<j<<endl;
				edges.push_back(edge);
				cafes[j].incoming.push_back(edge);
			}
		}
	}
	// "We build a sorted list of these connections (sorted by arriving time). 
	sort(edges.begin(), edges.end(), compare_edges);

	// "Iterate over the cornnections.
	// For each connection: if the node we are leaving has >1 inbound connection 
	// before we leave, choose the one with the greatest data size, update out 
	// outbound size accordingly."
	for (auto& e: edges) {
		e->data = 0;
		//cout<<"edge id = "<<e->id<<endl;
		// given my edge, find all edges that go TO the node I am coming FROM.
		if (e->from != -1 && cafes[e->from].incoming.size() > 0) {
			//cout << cafes[e->from].incoming.size()<<endl;
			// If they come to my node before I leave, pick the biggest data.
			// my data is the biggest data + the rate * seconds I stay at the node before I left.
			for(auto in: cafes[e->from].incoming) {
				//cout<<"in edge id = "<<in->id<<endl;
				uint64_t data = in->data + ((e->left - in->arrive) * cafes[e->from].w);
				//cout<<"in->data "<<in->data<<", e->left="<<e->left<<", in->arrive="<<in->arrive<<endl;
				//cout<<"data ="<<data<<endl;
				if (data > e->data && in->arrive <= e->left) {
					
				//cout<<"I arrived @ "<<e->to<<" from "<<e->from<<" at "<<e->arrive<<endl;
				//cout<<"I arrived at the previous at "<<in->arrive<<endl;
				//cout<<"in data ="<<in->data<<" e->left-in->arrive="<<(e->left - in->arrive)<<endl;
				//cout<<"data ="<<data<<endl;
					e->data = data;
				}
			}
		}
		//cout<<"## edge id = "<<e->id<<" e->data is now = "<<e->data<<endl;
	}

	int best = INT_MAX;
	for (auto e: edges) {
		// for each edge, we take the data at the start and assuem they stay where they are until the 
		// cae closes.
		uint64_t data = e->data + ((cafes[e->to].closes - e->arrive) * cafes[e->to].w);
		if (data >= d && e->arrive <= best) {
			// We may have a result.
			int s = ceil((float)(d - e->data) / (float)cafes[e->to].w);
			if (best > (s + e->arrive)) {
				best = s + e->arrive;
			}
		}
		//cout<<"cafe "<<e->to<<" closes @ "<<cafes[e->to].closes<<", I arrived from "<<e->from<<", @ "<<e->arrive<<", rate="<<cafes[e->to].w<<endl;
		//cout<<"best is now "<<best<<endl;
		//cout<<"cafe "<<e->to<<" e->data="<<e->data<<", max data="<<data<<endl;
	}
	cout<<(best == INT_MAX ? "impossible" : std::to_string(best))<<endl;
}
